@@ -1,20 +1,105 @@
 import json
+from time import sleep
 
-from gitlabform.gitlab.core import GitLabCore, NotFoundException
+from gitlabform.gitlab.core import (
+    GitLabCore,
+    NotFoundException,
+    TimeoutWaitingForDeletion,
+)
 
 
 class GitLabProjects(GitLabCore):
-    def create_project(self, name, path, namespace_id):
+    def get_project_case_insensitive(self, some_string):
+
+        # maybe "some_string" is the project's path
+
+        try:
+            return self.get_project(some_string)
+        except NotFoundException:
+
+            # try searching by name - maybe "some_string" is project's name
+            # or path, but case insensitive
+
+            projects = self._make_requests_to_api(
+                f"projects?search=%s&simple=true",
+                some_string,
+                method="GET",
+            )
+            for project in projects:
+                if (
+                    project["path"].lower() == some_string.lower()
+                    or project["name"].lower() == some_string.lower()
+                ):
+                    return project
+            raise NotFoundException
+
+    def create_project(
+        self,
+        name,
+        path,
+        namespace_id,
+        default_branch=None,
+        wait_if_still_being_deleted=False,
+    ):
         data = {
             "name": name,
             "path": path,
             "namespace_id": namespace_id,
         }
-        return self._make_requests_to_api(
-            "projects", data=data, method="POST", expected_codes=201
+        if default_branch:
+            data["default_branch"] = default_branch
+
+        if wait_if_still_being_deleted:
+
+            # GitLab deletes the project asynchronously, it may take a few seconds.
+            # So if you are creating new project with the same name as the one
+            # that is still being deleted, GitLab returns code 400
+            # and "The project is still being deleted.". Let's retry a few times
+            # then to start creating when the deletion is done.
+            # (Note: this code DOES NOT support the "Delayed Project deletion" feature
+            # where the actual deletion can be postponed for days!)
+
+            max_retries = 10
+            wait_before_retry = 3
+            retry = 0
+
+            while True:
+                retry += 1
+
+                if retry > max_retries:
+                    raise TimeoutWaitingForDeletion
+
+                response = self._make_requests_to_api(
+                    "projects", data=data, method="POST", expected_codes=[201, 400]
+                )
+                if self._is_project_still_deleted(response):
+                    # wait & retry
+                    sleep(wait_before_retry)
+                    continue
+                else:
+                    return response
+
+        else:
+
+            return self._make_requests_to_api(
+                "projects", data=data, method="POST", expected_codes=201
+            )
+
+    @staticmethod
+    def _is_project_still_deleted(response):
+        # check if response looks like this:
+        # {'message': {'base': ['The project is still being deleted. Please try again later.'],
+        # 'limit_reached': []}}
+        return (
+            "message" in response
+            and "base" in response["message"]
+            and type(response["message"]["base"]) == list
+            and len(response["message"]["base"]) == 1
+            and "The project is still being deleted." in response["message"]["base"][0]
         )
 
     def delete_project(self, project_and_group_name):
+
         # 404 means that the project does not exist anymore, so let's accept it for idempotency
         return self._make_requests_to_api(
             "projects/%s",
@@ -23,14 +108,25 @@ class GitLabProjects(GitLabCore):
             expected_codes=[202, 204, 404],
         )
 
-    def get_all_projects(self):
+        # GitLab deletes the project asynchronously, it may take a few seconds
+        # BUT it doesn't return such not yet deleted project in GET calls, so
+        # there is no point in checking if it actually done here. :(
+        # See create_project() for the code that deals with that.
+
+    def get_all_projects(self, include_archived=False):
         """
         :param group: group name
         :return: sorted list of ALL projects you have access to, strings "group/project_name"
         """
         try:
+            # there are 3 states of the "archived" flag: true, false, undefined
+            # we use the last 2
+            if include_archived:
+                query_string = "order_by=name&sort=asc"
+            else:
+                query_string = "order_by=name&sort=asc&archived=false"
             result = self._make_requests_to_api(
-                "projects?order_by=name&sort=asc", paginated=True
+                f"projects?{query_string}", paginated=True
             )
             return sorted(map(lambda x: x["path_with_namespace"], result))
         except NotFoundException:
@@ -326,6 +422,18 @@ class GitLabProjects(GitLabCore):
 
         return data
 
+    def get_groups_from_project(self, project_and_group_name):
+        # couldn't find an API call that was giving me directly
+        # the shared groups, so I'm using directly the GET /projects/:id call
+        project_info = self._make_requests_to_api("projects/%s", project_and_group_name)
+
+        # it will return {group_name: {...api info about group_name...}, ...}
+        groups = {}
+        for group in project_info["shared_with_groups"]:
+            groups[group["group_name"]] = group
+
+        return groups
+
     def share_with_group(
         self, project_and_group_name, group_name, group_access, expires_at
     ):
